#! /usr/bin/perl
use strict;
use warnings;
use Date::Manip;

# Parses tournament results from the DDR Europe result list

my $season = shift;
my ($name, $round, $group, $player, @rsongs);
die "Missing season (give on command line)." if (!defined($season));

# commented out -- see player below
# print "begin;\n";

while (<>) {
	if (/<h2> \s* (.*?) \s* <\/h2>/x) {
		$name = $1;
		next;
	}
	if (/<br>Country: \s* (.*?) \s*
             <br>Location: \s* (.*?) \s*
             <br>Date: \s* (.*?) \s*
             <br>Mix: \s* (.*?) \s*
             <br>ScoringSystem: \s* (.*?) \s* <br>
            /x) {
		my ($country, $location, $date, $mix, $system) = ($1, $2, $3, $4, $5);
		$mix =~ s/Euromix/EuroMix/;

		print "INSERT INTO tournaments \n";
		print "  (season, name, country, location, \"date\", machine, scoringsystem) VALUES (\n";
		print "    (SELECT season FROM seasons WHERE name='$season'),\n";
		print "    '$name',\n";
		print "    (SELECT country FROM countries WHERE name='$country'),\n";
		print "    '$location',\n";
		printf "    '%s',\n", Date::Manip::UnixDate($date, '%Y-%m-%d');
		print "    (SELECT machine FROM machines WHERE name='$mix'),\n";
		print "    (SELECT scoringsystem FROM scoringsystems WHERE name='$system')\n";
		print ");\n";
	}

	# New round
	if (/<h3 \s* class=dthr> Round \s* (\d+) \s* <\/h3>/x) {
		$round = $1;

		print "INSERT INTO rounds (tournament, round, randomsongs, chosensongs) \n";
		print "  VALUES (\n";
		print "   (SELECT tournament FROM tournaments WHERE name='$name'),\n";
		print "   $round,\n";
		print "   0, 0);\n";  # Don't worry, we'll fix it later :-P
	}
	
	# New group, and the random songs for it
	if (/<th \s* class=dthp> ( Players | Group \s* (\d+) )<\/th>/x) {
		# bit of an evil hack here
		$group = $2;
		$group = 0 if ($1 eq 'Players');

		print "INSERT INTO groups (tournament, round, parallel) VALUES (\n";
		print "   (SELECT tournament FROM tournaments WHERE name='$name'),\n";
		print "   $round, $group);\n";

		# Find the random songs, if any
		@rsongs = ();

		while (s/<th \s* class=dthpsong> .*? "color:white"> (.*?) <\/a><\/th>//x) {
			my $song = song_map($1);
			$song =~ s/'/\\'/g;
		
			push @rsongs, $song;

			print "INSERT INTO roundrandomsongs (tournament, round, parallel, song) VALUES (\n";
			print "   (SELECT tournament FROM tournaments WHERE name='$name'),\n";
			print "   $round, $group,\n";
			print "   (SELECT song FROM songs WHERE lower(title)=lower('$song'))\n";
			print ");\n";
		}

		# Correct the random songs in the table
		printf "UPDATE rounds SET randomsongs=%u WHERE \n", scalar @rsongs;
		print "  tournament=(SELECT tournament FROM tournaments WHERE name='$name') AND \n";
		print "  round=$round;\n";
	}

	# Header for chosen song (evil)
	if (/<th \s* class=dthp \s* colspan=2>&nbsp<\/th>/x) {
		printf "UPDATE rounds SET chosensongs=1 WHERE \n";
		print "  tournament=(SELECT tournament FROM tournaments WHERE name='$name') AND \n";
		print "  round=$round;\n";
	}

	# Player's results (header)
	if (/^ <tr><td \s* class=dtd> .*? class="link"> (.*?) <\/a> $/x) {
		$player = $1;
		printf "INSERT INTO players (nick) VALUES ('%s');\n", $player;
	}

	# Player's results (score)
	if (/<td \s* class=dtdm>/x) {
		my $i = 0;

		# random songs
		while (s/<td \s* class=dtdm> (\d+) <\/td>//x) {
			printf "INSERT INTO scores (tournament, round, parallel, player, song, chosen, score) VALUES (\n";
			print "   (SELECT tournament FROM tournaments WHERE name='$name'),\n";
			print "   $round, $group,\n";
			print "   (SELECT player FROM players WHERE nick='$player'),\n";
			print "   (SELECT song FROM songs WHERE lower(title)=lower('$rsongs[$i]')),\n";
			print "   'f',\n";
			print "   $1);\n";
			++$i;
		}

		# chosen songs
		while (s/<td \s* class=dtdsong> .*? class="link"> (.*?) <\/a> .*? <td class=dtdscore> (\d+) <\/td>//x) {
			my $song = song_map($1);
			$song =~ s/'/\\'/g;
		
			printf "INSERT INTO scores (tournament, round, parallel, player, song, chosen, score) VALUES (\n";
			print "   (SELECT tournament FROM tournaments WHERE name='$name'),\n";
			print "   $round, $group,\n";
			print "   (SELECT player FROM players WHERE nick='$player'),\n";
			print "   (SELECT song FROM songs WHERE lower(title)=lower('$song')),\n";
			print "   't',\n";
			print "   $2);\n";
		}
	}
}

printf "commit;\n";

# different names for songs
sub song_map {
	my $song = shift;

	$song =~ s/Paranoia -Rebirth-/PARANOiA Rebirth/;
	$song =~ s/The Center of the heart/The Centre of the Heart (Stonebridge Club Mix)/;
	$song =~ s/Can't stop falling in love/Can't Stop Fallin' in Love/;
	$song =~ s/B4U -B4 ZA Beat Mix-/B4U (B4 Za Beat Mix)/;
	$song =~ s/Jam Jam Reggae/Jam Jam Reggae (AM Swing Mix)/;
	$song =~ s/Trip Machine -Climax-/Trip Machine Climax/;
	$song =~ s/Dont try to stop it/Don't Try to Stop It/;
	$song =~ s/Healing Vision -AM-/Healing Vision (Angelic Mix)/;
	$song =~ s/5,6,7,8/5, 6, 7, 8/;
	$song =~ s/Keep On Movin/Keep On Movin'/;
	$song =~ s/So Deep/So Deep (Perfect Sphere Mix)/;
	$song =~ s/Aarons Party/Aaron's Party (Come Get It)/;

	return $song;
}
